@@ -57,8 +57,12 @@ GemmSize calculatePaddedGemmSize(const InitParams &params, GemmSize gemmSize,
 /// Given a tuning parameter struct, determine how much padding the gemm with
 /// a given gemm size requires. Returns None if no padding is needed. The
 /// values in the returned gemm context represent the number of 0s that need to
-/// be added to the given dimension.
-std::optional<GemmSize> requiredPadding(Attribute params, GemmSize gemmSize);
+/// be added to the given dimension. The mulBy* arguments multiply the
+/// corresponding dimension of the attributes.
+std::optional<GemmSize> requiredPadding(Attribute params, GemmSize gemmSize,
+                                        int64_t mulByKPerBlock = 1,
+                                        int64_t mulByMPerBlock = 1,
+                                        int64_t mulByNPerBlock = 1);
 
 int64_t obtainBlockSize(int64_t waveSize, int64_t mPerBlock, int64_t nPerBlock,
                         int64_t mPerWave, int64_t nPerWave);
@@ -101,24 +105,30 @@ struct InitParamsNonAccel : InitParams, Serializable<InitParamsNonAccel> {
   int64_t gemmNPerThread;
   uint32_t blockSize;
   int64_t splitKFactor;
+  int64_t gemmScheduleVersion;
+  int64_t outputSwizzle;
 
   constexpr InitParamsNonAccel(uint32_t bSize, int64_t mPerBlock,
                                int64_t nPerBlock, int64_t kPerBlock,
                                int64_t mPerThread, int64_t nPerThread,
-                               int64_t splitKFactor)
+                               int64_t splitKFactor, int64_t scheduleVersion,
+                               int64_t outputSwizzle)
       : InitParams{mPerBlock, nPerBlock, kPerBlock}, gemmMPerThread(mPerThread),
         gemmNPerThread(nPerThread), blockSize(bSize),
-        splitKFactor(splitKFactor) {}
+        splitKFactor(splitKFactor), gemmScheduleVersion(scheduleVersion),
+        outputSwizzle(outputSwizzle) {}
 
   constexpr InitParamsNonAccel()
-      : InitParamsNonAccel(0U, 0LL, 0LL, 0LL, 0LL, 0LL, 1LL) {}
+      : InitParamsNonAccel(0U, 0LL, 0LL, 0LL, 0LL, 0LL, 1LL, 1LL, 2LL) {}
 
   InitParamsNonAccel(GeneralGemmParamsAttr attr)
       : InitParams{attr.getMPerBlock(), attr.getNPerBlock(),
                    attr.getKPerBlock()},
         gemmMPerThread(attr.getMPerThread()),
         gemmNPerThread(attr.getNPerThread()), blockSize(attr.getBlockSize()),
-        splitKFactor(attr.getSplitKFactor()) {};
+        splitKFactor(attr.getSplitKFactor()),
+        gemmScheduleVersion(attr.getScheduleVersion()),
+        outputSwizzle(attr.getOutputSwizzle()) {};
 
   int64_t getKPack() { return 1; }
 
@@ -130,8 +140,12 @@ struct InitParamsNonAccel : InitParams, Serializable<InitParamsNonAccel> {
     f(self.gemmKPerBlock);
     f(self.gemmMPerThread);
     f(self.gemmNPerThread);
-    if (self.version != Version::V1)
+    if (self.version >= Version::V2)
       f(self.splitKFactor);
+    if (self.version >= Version::V3) {
+      f(self.gemmScheduleVersion);
+      f(self.outputSwizzle);
+    }
   }
 };
 
@@ -139,16 +153,19 @@ struct InitParamsAccel : InitParams, Serializable<InitParamsAccel> {
   constexpr InitParamsAccel(int64_t mPerBlock, int64_t nPerBlock,
                             int64_t kPerBlock, int64_t mPerWave,
                             int64_t nPerWaveOrMnPerXdl, int64_t kPack,
-                            int64_t splitKFactor, bool aThreadCopyMoreGemmK,
+                            int64_t splitKFactor, int64_t scheduleVersion,
+                            int64_t outputSwizzle, bool aThreadCopyMoreGemmK,
                             bool bThreadCopyMoreGemmKPack)
       : InitParams{mPerBlock, nPerBlock, kPerBlock}, gemmMPerWave(mPerWave),
         gemmNPerWaveOrMnPerXdl(nPerWaveOrMnPerXdl), gemmKPack(kPack),
-        splitKFactor(splitKFactor),
+        splitKFactor(splitKFactor), gemmScheduleVersion(scheduleVersion),
+        outputSwizzle(outputSwizzle),
         gemmAThreadCopyMoreGemmK(aThreadCopyMoreGemmK),
         gemmBThreadCopyMoreGemmKPack(bThreadCopyMoreGemmKPack) {}
 
   constexpr InitParamsAccel()
-      : InitParamsAccel(0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 1LL, false, false) {}
+      : InitParamsAccel(0LL, 0LL, 0LL, 0LL, 0LL, 0LL, 1LL, 1LL, 2LL, false,
+                        false) {}
 
   InitParamsAccel(XdlopsGemmParamsAttr attr)
       : InitParams{attr.getMPerBlock(), attr.getNPerBlock(),
@@ -156,6 +173,8 @@ struct InitParamsAccel : InitParams, Serializable<InitParamsAccel> {
         gemmMPerWave(attr.getMPerWave()),
         gemmNPerWaveOrMnPerXdl(attr.getMnPerXdl()), gemmKPack(attr.getKpack()),
         splitKFactor(attr.getSplitKFactor()),
+        gemmScheduleVersion(attr.getScheduleVersion()),
+        outputSwizzle(attr.getOutputSwizzle()),
         gemmAThreadCopyMoreGemmK(attr.getForceUnroll()),
         gemmBThreadCopyMoreGemmKPack(false) {};
 
@@ -165,6 +184,8 @@ struct InitParamsAccel : InitParams, Serializable<InitParamsAccel> {
         gemmMPerWave(attr.getMPerWave()),
         gemmNPerWaveOrMnPerXdl(attr.getNPerWave()), gemmKPack(attr.getKpack()),
         splitKFactor(attr.getSplitKFactor()),
+        gemmScheduleVersion(attr.getScheduleVersion()),
+        outputSwizzle(attr.getOutputSwizzle()),
         gemmAThreadCopyMoreGemmK(attr.getForceUnroll()),
         gemmBThreadCopyMoreGemmKPack(false) {};
 
@@ -174,6 +195,8 @@ struct InitParamsAccel : InitParams, Serializable<InitParamsAccel> {
   int64_t gemmNPerWaveOrMnPerXdl;
   int64_t gemmKPack;
   int64_t splitKFactor;
+  int64_t gemmScheduleVersion;
+  int64_t outputSwizzle;
   bool gemmAThreadCopyMoreGemmK;
   bool gemmBThreadCopyMoreGemmKPack;
 
@@ -185,9 +208,13 @@ struct InitParamsAccel : InitParams, Serializable<InitParamsAccel> {
     f(self.gemmMPerWave);
     f(self.gemmNPerWaveOrMnPerXdl);
     f(self.gemmKPack);
-    if (self.version != Version::V1) {
+    if (self.version >= Version::V2) {
       f(self.splitKFactor);
     }
+    if (self.version >= Version::V3) {
+      f(self.gemmScheduleVersion);
+      f(self.outputSwizzle);
+    }
     f(self.gemmAThreadCopyMoreGemmK);
     f(self.gemmBThreadCopyMoreGemmKPack);
   }
@@ -206,14 +233,15 @@ class BasePopulateParams {
 private:
   struct InitParamData {
     InitParamType paramSet;
-    size_t original_pos;
-    int64_t padding_amount;
+    size_t originalPos;
+    int64_t paddingAmount;
 
     bool operator<(const InitParamData &rhs) const {
-      if (this->padding_amount < rhs.padding_amount) {
+      if (this->paddingAmount < rhs.paddingAmount) {
         return true;
-      } else if (this->padding_amount == rhs.padding_amount) {
-        return (this->original_pos < rhs.original_pos);
+      }
+      if (this->paddingAmount == rhs.paddingAmount) {
+        return (this->originalPos < rhs.originalPos);
       }
       return false;
     }
@@ -227,10 +255,10 @@ private:
       InitParamType paramSet = initParams[pos];
       InitParamData paramData;
       paramData.paramSet = paramSet;
-      paramData.original_pos = pos;
-      paramData.padding_amount = calculatePaddingAmount(paramSet, gemmSize);
-      assert(paramData.original_pos >= 0);
-      assert(paramData.padding_amount >= 0);
+      paramData.originalPos = pos;
+      paramData.paddingAmount = calculatePaddingAmount(paramSet, gemmSize);
+      assert(paramData.originalPos >= 0);
+      assert(paramData.paddingAmount >= 0);
       res.push_back(paramData);
     }
     return res;
@@ -309,8 +337,10 @@ public:
 
   // Return the vector of heuristic parameters for a given kernel type and dat
   // type.
-  std::vector<InitParamsNonAccel>
-  getTuningParameters(KernelType opType, Type dataTypeA, Type dataTypeB) const;
+  std::vector<InitParamsNonAccel> getTuningParameters(KernelType opType,
+                                                      Type dataTypeA,
+                                                      Type dataTypeB,
+                                                      StringRef arch) const;
 
   Attribute getGemmParamsAttr(OpBuilder &b,
                               const InitParamsNonAccel &params) const override;
