@@ -32,9 +32,10 @@
 #include "mlir/Dialect/LLVMIR/LLVMDialect.h"
 #include "mlir/Dialect/MHAL/IR/MHAL.h"
 #include "mlir/Dialect/MemRef/IR/MemRef.h"
+#include "mlir/Dialect/Rock/IR/AmdArchDb.h"
+#include "mlir/Dialect/Rock/IR/GetRockInfo.h"
 #include "mlir/Dialect/Rock/IR/Rock.h"
 #include "mlir/Dialect/Rock/Passes.h"
-#include "mlir/Dialect/Rock/utility/AmdArchDb.h"
 #include "mlir/Dialect/Rock/utility/loweringUtils.h"
 #include "mlir/IR/BuiltinAttributes.h"
 #include "mlir/IR/BuiltinOps.h"
@@ -118,7 +119,7 @@ struct MIOpRewritePattern : public OpRewritePattern<Tmi> {
   using OpRewritePattern<Tmi>::OpRewritePattern;
 
   LogicalResult matchAndRewrite(Tmi op, PatternRewriter &b) const override {
-    b.create<Tgpu>(op.getLoc());
+    Tgpu::create(b, op.getLoc());
     op.erase();
     return success();
   }
@@ -140,24 +141,8 @@ struct WorkgroupIdRewritePattern
 
   LogicalResult matchAndRewrite(rock::WorkgroupIdOp op,
                                 PatternRewriter &b) const override {
-    Location loc = op.getLoc();
-    auto maybeIsReverseGrid = rock::getReverseGrid(op);
-    if (succeeded(maybeIsReverseGrid)) {
-      FailureOr<IntegerAttr> maybeGridSize = rock::getGridSize(op);
-      if (failed(maybeGridSize)) {
-        return op->emitError("grid_size should ve been set by now.\n");
-      }
-      int64_t gridSize = maybeGridSize.value().getValue().getSExtValue();
-      AffineMap reverseMap = rock::getIdxReversalMap(b);
-      Value gridSizeVal = b.createOrFold<arith::ConstantIndexOp>(loc, gridSize);
-      Value blockIdVal = b.create<gpu::BlockIdOp>(loc, gpu::Dimension::x);
-      b.replaceOpWithNewOp<affine::AffineApplyOp>(
-          op, b.getIndexType(), reverseMap,
-          ValueRange{blockIdVal, gridSizeVal});
-    } else {
-      b.replaceOpWithNewOp<gpu::BlockIdOp>(op, b.getIndexType(),
-                                           gpu::Dimension::x);
-    }
+    b.replaceOpWithNewOp<gpu::BlockIdOp>(op, b.getIndexType(),
+                                         gpu::Dimension::x);
     return success();
   }
 };
@@ -175,7 +160,7 @@ void LowerRockOpsToGPUPass::runOnOperation() {
 
   auto makeGpuModule = [&](StringRef name) {
     // create a GPUModuleOp in case the GPU module specified does not exist.
-    auto gpuModule = b.create<gpu::GPUModuleOp>(loc, name);
+    auto gpuModule = gpu::GPUModuleOp::create(b, loc, name);
 
     // add the GPUModuleOp into the symbol table.
     SymbolTable symbolTable(op);
@@ -195,7 +180,7 @@ void LowerRockOpsToGPUPass::runOnOperation() {
     // create a GPUFuncOp.
     FunctionType gpuFuncType = theFunc.getFunctionType();
     auto gpuFunc =
-        b.create<gpu::GPUFuncOp>(loc, theFunc.getName(), gpuFuncType);
+        gpu::GPUFuncOp::create(b, loc, theFunc.getName(), gpuFuncType);
 
     // insert the GPUFuncOp into GPUModuleOp.
     gpuModuleSymbolTable.insert(gpuFunc);
@@ -220,9 +205,6 @@ void LowerRockOpsToGPUPass::runOnOperation() {
       gridSize = cast<IntegerAttr>(attr).getInt();
       gpuFunc.setKnownGridSizeAttr(b.getDenseI32ArrayAttr({gridSize, 1, 1}));
     }
-    if (auto isReverse = rock::getReverseGrid(theFunc).value_or(nullptr)) {
-      gpuFunc->setAttr(rock::ReverseGridAttrAttr::getMnemonic(), isReverse);
-    }
     FailureOr<StringAttr> maybeArch = rock::getArch(theFunc);
     if (succeeded(maybeArch)) {
       gpuFunc->setAttr("arch", maybeArch.value());
@@ -268,7 +250,7 @@ void LowerRockOpsToGPUPass::runOnOperation() {
     Block *clonedFuncEntry = map.lookup(&funcEntry);
     Block &gpuFuncEntry = gpuFuncBody.front();
     b.setInsertionPointToEnd(&gpuFuncEntry);
-    b.create<cf::BranchOp>(loc, clonedFuncEntry);
+    cf::BranchOp::create(b, loc, clonedFuncEntry);
 
     // Ask LLVM to use atomic intrinsics instead of generic CAS loops whenever
     // it can
@@ -313,16 +295,16 @@ void LowerRockOpsToGPUPass::runOnOperation() {
                 mlir::dyn_cast<SymbolRefAttr>(callable))) {
           if (symRef.getValue() == theFunc.getName()) {
             OpBuilder b(call);
-            auto gridVal = b.create<arith::ConstantIndexOp>(loc, gridSize);
-            auto blockVal = b.create<arith::ConstantIndexOp>(loc, blockSize);
-            auto cst1 = b.create<arith::ConstantIndexOp>(loc, 1);
+            auto gridVal = arith::ConstantIndexOp::create(b, loc, gridSize);
+            auto blockVal = arith::ConstantIndexOp::create(b, loc, blockSize);
+            auto cst1 = arith::ConstantIndexOp::create(b, loc, 1);
             auto dynamicSharedMemSize =
-                b.create<arith::ConstantIntOp>(loc, 0, b.getI32Type());
+                arith::ConstantIntOp::create(b, loc, b.getI32Type(), 0);
             gpu::KernelDim3 gridDims{gridVal, cst1, cst1};
             gpu::KernelDim3 blockDims{blockVal, cst1, cst1};
-            b.create<gpu::LaunchFuncOp>(loc, gpuFunc, gridDims, blockDims,
-                                        dynamicSharedMemSize,
-                                        call.getArgOperands());
+            gpu::LaunchFuncOp::create(b, loc, gpuFunc, gridDims, blockDims,
+                                      dynamicSharedMemSize,
+                                      call.getArgOperands());
             calls.push_back(call);
           }
         }
