@@ -7,7 +7,9 @@
 //===-----------------------------------------------------===//
 
 #include "mlir/Dialect/Rock/utility/loweringUtils.h"
-#include "mlir/Dialect/Rock/utility/AmdArchDb.h"
+#include "mlir/Dialect/Linalg/IR/Linalg.h"
+#include "mlir/Dialect/Rock/IR/AmdArchDb.h"
+#include "mlir/Dialect/Rock/IR/GetRockInfo.h"
 #include "mlir/Dialect/Rock/utility/builderUtils.h"
 #include "mlir/Dialect/Rock/utility/transformMapUtils.h"
 
@@ -18,6 +20,9 @@
 #include "mlir/Dialect/Rock/utility/math.h"
 #include "mlir/IR/BuiltinAttributes.h"
 #include "mlir/IR/Matchers.h"
+#include "mlir/IR/Value.h"
+#include "mlir/Support/LLVM.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/ErrorHandling.h"
 #include "llvm/Support/FormatVariadic.h"
@@ -40,10 +45,6 @@ bool mlir::rock::isWrWAtomicKernel(GemmFeatures features, Type dataType,
          (dataType.isF32() || dataType.isF16()) && !requiredPadding;
 }
 
-bool mlir::rock::isAccel(GemmFeatures features) {
-  return bitEnumContainsAny(features, GemmFeatures::wmma | GemmFeatures::mfma);
-}
-
 bool mlir::rock::is4GBMemoryType(ShapedType type) {
   if (!type.hasStaticShape())
     return true;
@@ -104,10 +105,22 @@ LogicalResult mlir::rock::calculateKBlockNum(const int64_t batchSize,
   return success();
 }
 
+bool mlir::rock::isEveryElementWrittenBwdData(ArrayRef<int64_t> strideDims,
+                                              ArrayRef<int64_t> dilationDims,
+                                              ArrayRef<int64_t> filterDims) {
+  bool result = true;
+  for (const auto &[stride, dilation, filterSize] :
+       zip(strideDims, dilationDims, filterDims)) {
+    if (!(dilation == 1 && stride <= filterSize))
+      result = false;
+  }
+  return result;
+}
+
 SmallVector<int64_t>
 mlir::rock::backwardDataKernelIds(ArrayRef<int64_t> strideDims,
                                   ArrayRef<int64_t> dilationDims,
-                                  ArrayRef<int64_t> filterDims) {
+                                  ArrayRef<int64_t> filterDims, bool usesV4R1) {
   assert(strideDims.size() == dilationDims.size());
   SmallVector<int64_t, 5> gcdStrideDilations;
   for (const auto &[stride, dilation] : zip(strideDims, dilationDims))
@@ -117,25 +130,9 @@ mlir::rock::backwardDataKernelIds(ArrayRef<int64_t> strideDims,
   for (const auto &[stride, gcdSD] : zip(strideDims, gcdStrideDilations))
     filTilda.push_back(stride / gcdSD);
 
-  // Heuristic to determine if every pixel in the output would be written by the
-  // backward data convolution algorithm.
-  auto isEveryPixelWritten = [&]() -> bool {
-    bool result = true;
-    for (const auto &[stride, dilation, filterSize] :
-         zip(strideDims, dilationDims, filterDims)) {
-      if (!(dilation == 1 && stride <= filterSize))
-        result = false;
-    }
-    return result;
-  };
-  bool needZeroInitKernel = !isEveryPixelWritten();
-
-  llvm::SmallVector<int64_t> kernelIds;
-  if (needZeroInitKernel)
-    kernelIds.push_back(-1);
-
   // Populate the kernel IDs according to the current backward data convolution
   // algorithm implementation.
+  llvm::SmallVector<int64_t> kernelIds;
   int64_t subproduct = 1;
   int64_t product;
   for (size_t i = 1; i < filterDims.size(); i++)
@@ -404,7 +401,22 @@ Value mlir::rock::normalizeMatrix(Value matrix, OpBuilder &b, Location loc,
 
   normalizer.passThrough({firstDim, secondDim}, {1, 2}, {firstDim, secondDim});
   TransformMapAttr normalizeAttr = normalizer.get();
-  return b.create<TransformOp>(loc, matrix, normalizeAttr);
+  return TransformOp::create(b, loc, matrix, normalizeAttr);
+}
+
+Value mlir::rock::padVector(Value vector, OpBuilder &b, Location loc,
+                            StringRef firstDim, int64_t firstDimPad) {
+  if (firstDimPad == 0)
+    return vector;
+  ArrayRef<int64_t> shape = cast<MemRefType>(vector.getType()).getShape();
+  assert(shape.size() == 2);
+  BottomUpTMBuilder padder(b, {"gemmG", firstDim}, shape, loc);
+  padder.passThrough("gemmG");
+  SmallString<8> paddedName;
+  (firstDim + Twine("Pad")).toVector(paddedName);
+  padder.pad(paddedName, firstDim, 0, firstDimPad);
+  TransformMapAttr padAttr = padder.get();
+  return TransformOp::create(b, loc, vector, padAttr);
 }
 
 Value mlir::rock::padMatrix(Value matrix, OpBuilder &b, Location loc,
@@ -430,7 +442,7 @@ Value mlir::rock::padMatrix(Value matrix, OpBuilder &b, Location loc,
     padder.pad(paddedName, secondDim, 0, secondDimPad);
   }
   TransformMapAttr padAttr = padder.get();
-  return b.create<TransformOp>(loc, matrix, padAttr);
+  return TransformOp::create(b, loc, matrix, padAttr);
 }
 
 TopDownTMBuilder mlir::rock::swapThreadIdAndIteration(
@@ -527,8 +539,8 @@ Value mlir::rock::createSliceOfFirstDim(PatternRewriter &rewriter, Location loc,
   auto dstMemref =
       cast<MemRefType>(memref::SubViewOp::inferRankReducedResultType(
           originalShape, bufType, offsets, sizes, strides));
-  Value subview = rewriter.create<memref::SubViewOp>(loc, dstMemref, buffer,
-                                                     offsets, sizes, strides);
+  Value subview = memref::SubViewOp::create(rewriter, loc, dstMemref, buffer,
+                                            offsets, sizes, strides);
   return subview;
 }
 
@@ -575,7 +587,7 @@ FailureOr<memref::AllocOp> mlir::rock::findMemrefAlloc(Value value) {
   return findAlloc<memref::AllocOp>(value);
 }
 
-FailureOr<BlockArgument> findBlockArgument(Value value) {
+FailureOr<BlockArgument> mlir::rock::findBlockArgument(Value value) {
   auto maybeBlockArg = dyn_cast_or_null<BlockArgument>(value);
   while (!maybeBlockArg) {
     // Keep going until the operation that defines the value is a
@@ -661,20 +673,13 @@ mlir::rock::transposeSubTileViews(PatternRewriter &rewriter, Location loc,
   }
 }
 
+// Helper function to get attributes from parents
 template <typename RetAttrType>
-static FailureOr<RetAttrType> getAttrFromOpOrParents(
+FailureOr<RetAttrType> getAttrFromOpOrParents(
     Operation *op, StringRef opAttr,
     std::optional<StringRef> maybeDialectAttr = std::nullopt) {
   StringRef dialectAttr = maybeDialectAttr.value_or(opAttr);
-  Operation *func;
-  if (isa<func::FuncOp, gpu::GPUFuncOp>(op)) {
-    func = op;
-  } else {
-    func = op->getParentOfType<func::FuncOp>();
-    if (!func) {
-      func = op->getParentOfType<gpu::GPUFuncOp>();
-    }
-  }
+  Operation *func = getParentFuncOp(op);
   RetAttrType attr;
   auto getAnyAttr = [&](ArrayRef<StringRef> attrNames, Operation *op) {
     for (StringRef attrName : attrNames) {
@@ -685,55 +690,30 @@ static FailureOr<RetAttrType> getAttrFromOpOrParents(
       }
     }
   };
+
+  // First check for the attribute on the op
   getAnyAttr({opAttr}, op);
   if (!attr) {
+    // If that fails then try checking for the attribute on the func
     getAnyAttr({opAttr, dialectAttr}, func);
   }
+
+  // If there is no desired attribute on the func, then check the nearest parent
+  // with a symbol table (covers both ModuleOp and gpu::GPUModuleOp)
   if (!attr) {
-    auto mod = func->getParentOfType<ModuleOp>();
-    getAnyAttr({opAttr, dialectAttr}, mod);
-  }
-  if (!attr) {
-    if (auto mod = func->getParentOfType<gpu::GPUModuleOp>()) {
-      getAnyAttr({opAttr, dialectAttr}, mod);
+    if (auto symbolTableOp = func->getParentWithTrait<OpTrait::SymbolTable>()) {
+      getAnyAttr({opAttr, dialectAttr}, symbolTableOp);
+      if (attr)
+        return attr;
     }
   }
+
   if (!attr) {
     return failure();
   }
   return attr;
 }
 
-FailureOr<StringAttr> mlir::rock::getArch(Operation *op) {
-  return getAttrFromOpOrParents<StringAttr>(op, "arch", "mhal.arch");
-}
-
-FailureOr<int64_t> mlir::rock::getNumCU(Operation *op) {
-  FailureOr<StringAttr> maybeArch = getArch(op);
-  if (failed(maybeArch)) {
-    return failure();
-  }
-  StringAttr arch = maybeArch.value();
-  FailureOr<IntegerAttr> maybeNumCU =
-      getAttrFromOpOrParents<IntegerAttr>(op, "num_cu");
-  if (failed(maybeNumCU)) {
-    return failure();
-  }
-  IntegerAttr numCU = maybeNumCU.value();
-  AmdArchInfo archInfo = rock::lookupArchInfo(arch);
-  if (numCU.getValue().getSExtValue() < archInfo.minNumCU) {
-    return op->emitError() << "num_cu=" << numCU
-                           << " cannot be lower than arch minNumCU="
-                           << archInfo.minNumCU;
-  }
-  return numCU.getValue().getSExtValue();
-}
-
-FailureOr<UnitAttr> mlir::rock::getReverseGrid(Operation *op) {
-  return getAttrFromOpOrParents<UnitAttr>(
-      op, rock::ReverseGridAttrAttr::getMnemonic());
-}
-
 FailureOr<IntegerAttr> mlir::rock::getGridSize(Operation *op) {
   return getAttrFromOpOrParents<IntegerAttr>(op, "grid_size");
 }
@@ -742,13 +722,6 @@ FailureOr<IntegerAttr> mlir::rock::getBlockSize(Operation *op) {
   return getAttrFromOpOrParents<IntegerAttr>(op, "block_size");
 }
 
-AffineMap mlir::rock::getIdxReversalMap(OpBuilder &b) {
-  auto dimExpr = mlir::getAffineDimExpr(0, b.getContext());
-  auto dimSizeExpr = mlir::getAffineSymbolExpr(0, b.getContext());
-  auto affineMap = mlir::AffineMap::get(1, 1, dimSizeExpr - 1 - dimExpr);
-  return affineMap;
-}
-
 ReassociationIndices
 mlir::rock::getReassociationForFlattening(ShapedType srcTp) {
   ReassociationIndices reassociation;
@@ -757,19 +730,37 @@ mlir::rock::getReassociationForFlattening(ShapedType srcTp) {
   return reassociation;
 }
 
+Value mlir::rock::getFlattenedMemref(OpBuilder &b, Value nonFlatMemRef) {
+  Location loc = nonFlatMemRef.getLoc();
+  MemRefType nonFlatMemRefType = cast<MemRefType>(nonFlatMemRef.getType());
+  int64_t numElements = nonFlatMemRefType.getNumElements();
+  if (nonFlatMemRefType.getRank() > 1) {
+    Type nonFlatMemRefElType = nonFlatMemRefType.getElementType();
+    auto flatMemRefType =
+        MemRefType::get({numElements}, nonFlatMemRefElType, AffineMap{},
+                        nonFlatMemRefType.getMemorySpace());
+    auto reassociation = getReassociationForFlattening(nonFlatMemRefType);
+    return memref::CollapseShapeOp::create(b, loc, flatMemRefType,
+                                           nonFlatMemRef, reassociation);
+  }
+  return nonFlatMemRef;
+}
+
 TypedValue<MemRefType> mlir::rock::viewBufferAs(OpBuilder &b, Value buffer,
                                                 Type type) {
   Location loc = buffer.getLoc();
   Value zeroByteOffset = b.createOrFold<arith::ConstantIndexOp>(loc, 0);
   auto bufferType = cast<MemRefType>(buffer.getType());
   int64_t byteWidth = getByteWidth(type);
+  assert(bufferType.getRank() == 1 &&
+         "Buffer type must be a 1D memref for viewBufferAs");
   int64_t numBytes = bufferType.getShape()[0];
   assert(numBytes % byteWidth == 0 && "Can't evenly fit type into buffer");
   int64_t length = numBytes / byteWidth;
   auto newBufferType = bufferType.cloneWith({length}, type);
   auto view =
-      b.create<memref::ViewOp>(loc, newBufferType, buffer, zeroByteOffset,
-                               /*dynamic dim sizes=*/ValueRange{});
+      memref::ViewOp::create(b, loc, newBufferType, buffer, zeroByteOffset,
+                             /*dynamic dim sizes=*/ValueRange{});
   return TypedValue<MemRefType>(view.getResult());
 }
 
@@ -785,12 +776,12 @@ Value mlir::rock::gpuAlloc(OpBuilder &b, Location loc, int64_t bufferDim,
   if (memoryAddressSpace == gpu::AddressSpace::Private) {
     auto memType = MemRefType::get({bufferDim}, elementType, AffineMap{},
                                    memoryAddressSpaceAttr);
-    return b.create<GpuAllocOp>(loc, memType);
+    return GpuAllocOp::create(b, loc, memType);
   }
   auto rawMemType =
       MemRefType::get({bufferDim * getByteWidth(elementType)}, b.getI8Type(),
                       AffineMap{}, memoryAddressSpaceAttr);
-  auto buffer = b.create<GpuAllocOp>(loc, rawMemType);
+  auto buffer = GpuAllocOp::create(b, loc, rawMemType);
 
   return viewBufferAs(b, buffer, elementType);
 }
@@ -801,10 +792,10 @@ LogicalResult mlir::rock::checkLDSSize(StringAttr arch, int64_t ldsBytes) {
   return success(ldsBytes <= ldsSize);
 }
 
-// Trace an arg back to its alloc.
-static void traceGemmAllocToArgs(memref::AllocOp buffer,
-                                 const BufferDependencyAnalysis &deps,
-                                 SmallVector<BlockArgument> &args) {
+static void traceAlloc(memref::AllocOp buffer,
+                       const BufferDependencyAnalysis &deps,
+                       SmallVector<BlockArgument> &args,
+                       SmallVector<OpOperand *> &genericOpOperands) {
   IRRewriter rewriter(buffer.getContext());
   std::optional<llvm::SmallVector<OpOperand *>> readersOperands =
       deps.getReaders(buffer);
@@ -815,6 +806,9 @@ static void traceGemmAllocToArgs(memref::AllocOp buffer,
     if (!readOp)
       continue;
 
+    if (auto genericOp = dyn_cast<linalg::GenericOp>(readerOperand->getOwner()))
+      genericOpOperands.push_back(readerOperand);
+
     SmallVector<MemoryEffects::EffectInstance> effects;
     readOp.getEffects(effects);
     for (const MemoryEffects::EffectInstance &effect : effects) {
@@ -823,11 +817,14 @@ static void traceGemmAllocToArgs(memref::AllocOp buffer,
       if (writerOperand && readerOperand != writerOperand &&
           isa<MemoryEffects::Write>(effect.getEffect())) {
         Value writerOperandValue = writerOperand->get();
-        if (auto blockArg = dyn_cast<BlockArgument>(writerOperandValue))
-          args.push_back(blockArg);
-        else if (memref::AllocOp writeBuffer = dyn_cast<memref::AllocOp>(
-                     writerOperandValue.getDefiningOp()))
-          traceGemmAllocToArgs(writeBuffer, deps, args);
+
+        FailureOr<BlockArgument> maybeArg =
+            findBlockArgument(writerOperandValue);
+        if (succeeded(maybeArg))
+          args.push_back(maybeArg.value());
+        else if (memref::AllocOp writeBuffer =
+                     writerOperandValue.getDefiningOp<memref::AllocOp>())
+          traceAlloc(writeBuffer, deps, args, genericOpOperands);
       }
     }
   }
@@ -856,7 +853,8 @@ mlir::rock::traceGemmOutputToArgs(Value matC, func::FuncOp func,
     return failure();
 
   // trace gemm alloc to arg
-  traceGemmAllocToArgs(allocOp.value(), deps, args);
+  SmallVector<OpOperand *> genericOpOperands;
+  traceAlloc(allocOp.value(), deps, args, genericOpOperands);
   for (auto arg : args) {
     bool containsArg =
         std::find(funcArgs.begin(), funcArgs.end(), arg) != funcArgs.end();
@@ -868,3 +866,27 @@ mlir::rock::traceGemmOutputToArgs(Value matC, func::FuncOp func,
 
   return failure();
 }
+
+FailureOr<SmallVector<OpOperand *>>
+mlir::rock::traceGemmOutputToGenericOps(Value matC, func::FuncOp func,
+                                        const BufferDependencyAnalysis &deps) {
+  auto funcArgs = func.getArguments();
+  // check if matC is a kernel argument
+  for (auto arg : funcArgs) {
+    // no possible linalg.generic output fusion if matC is a block arg
+    if (findBlockArgument(matC) == arg)
+      return {};
+  }
+
+  // trace matC to its alloc
+  FailureOr<memref::AllocOp> allocOp = findMemrefAlloc(matC);
+  if (failed(allocOp))
+    return failure();
+
+  // trace gemm alloc to arg, saving all genericOps
+  SmallVector<OpOperand *> genericOpOperands;
+  SmallVector<BlockArgument> args;
+  traceAlloc(allocOp.value(), deps, args, genericOpOperands);
+
+  return genericOpOperands;
+}
